/*
 *                      EBUILDA: cross-platform build system
 *
 * This file is part of Yeppp! library infrastructure and licensed under Simplified BSD License.
 * See LICENSE.txt for the full text of the license.
 *
 */

package info.yeppp.ebuilda;

public enum ABI {

	X86_Linux_Pic_i586,
	X86_Linux_Pic_Android,
	X86_MacOSX_Pic_Default,
	X86_Windows_Default_i586,
	X86_NaCl_NaCl_i686,

	X64_Linux_SystemV_Default,
	X64_Linux_KNC_Default,
	X64_MacOSX_SystemV_Default,
	X64_Windows_Microsoft_Default,
	X64_NaCl_NaCl_Default,

	ARM_Linux_OABI_V4T,
	ARM_Linux_SoftEABI_V5T,
	ARM_Linux_HardEABI_V7A,
	ARM_Linux_SoftEABI_Android,
	ARM_Linux_SoftEABI_AndroidV7A,
	ARM_Windows_HardEABI_Default,
	ARM_NaCl_NaCl_Default,

	MIPS_Linux_O32_Android,

	IA64_Windows_Microsoft_Default;

	public static ABI parse(String string) {
		if (string.equals("x86-linux-pic-i586")) {
			return X86_Linux_Pic_i586;
		} else if (string.equals("x86-linux-pic-android")) {
			return X86_Linux_Pic_Android;
		} else if (string.equals("x86-macosx-pic-default")) {
			return X86_MacOSX_Pic_Default;
		} else if (string.equals("x86-windows-default-i586")) {
			return X86_Windows_Default_i586;
		} else if (string.equals("x86-nacl-pic-i686")) {
			return X86_NaCl_NaCl_i686;
		} else if (string.equals("x64-linux-sysv-default")) {
			return X64_Linux_SystemV_Default;
		} else if (string.equals("x64-linux-k1om-default")) {
			return X64_Linux_KNC_Default;
		} else if (string.equals("x64-macosx-sysv-default")) {
		    return X64_MacOSX_SystemV_Default;
		} else if (string.equals("x64-windows-ms-default")) {
			return X64_Windows_Microsoft_Default;
		} else if (string.equals("x64-nacl-nacl-default")) {
			return X64_NaCl_NaCl_Default;
		} else if (string.equals("arm-linux-oabi-v4t")) {
			return ARM_Linux_OABI_V4T;
		} else if (string.equals("arm-linux-softeabi-v5t")) {
			return ARM_Linux_SoftEABI_V5T;
		} else if (string.equals("arm-linux-hardeabi-v7a")) {
			return ARM_Linux_HardEABI_V7A;
		} else if (string.equals("arm-linux-softeabi-android")) {
			return ARM_Linux_SoftEABI_Android;
		} else if (string.equals("arm-linux-softeabi-androidv7a")) {
			return ARM_Linux_SoftEABI_AndroidV7A;
		} else if (string.equals("arm-windows-hardeabi-default")) {
			return ARM_Windows_HardEABI_Default;
		} else if (string.equals("arm-nacl-nacl-default")) {
			return ARM_NaCl_NaCl_Default;
		} else if (string.equals("mips-linux-o32-android")) {
			return MIPS_Linux_O32_Android;
		} else if (string.equals("ia64-windows-ms-default")) {
			return IA64_Windows_Microsoft_Default;
		} else {
			throw new IllegalArgumentException(String.format("Could not parse string \"%s\" as ABI name", string));
		}
	}

	@Override
	public String toString() {
		switch (this) {
			case X86_Linux_Pic_i586:
				return "x86-linux-pic-i586";
			case X86_Linux_Pic_Android:
				return "x86-linux-pic-android";
			case X86_MacOSX_Pic_Default:
				return "x86-macosx-pic-default";
			case X86_Windows_Default_i586:
				return "x86-windows-default-i586";
			case X86_NaCl_NaCl_i686:
				return "x86-nacl-pic-i686";
			case X64_Linux_SystemV_Default:
				return "x64-linux-sysv-default";
			case X64_Linux_KNC_Default:
				return "x64-linux-k1om-default";
			case X64_MacOSX_SystemV_Default:
				return "x64-macosx-sysv-default";
			case X64_Windows_Microsoft_Default:
				return "x64-windows-ms-default";
			case X64_NaCl_NaCl_Default:
				return "x64-nacl-sysv-default";
			case ARM_Linux_OABI_V4T:
				return "arm-linux-oabi-v4t";
			case ARM_Linux_SoftEABI_V5T:
				return "arm-linux-softeabi-v5t";
			case ARM_Linux_HardEABI_V7A:
				return "arm-linux-hardeabi-v7a";
			case ARM_Linux_SoftEABI_Android:
				return "arm-linux-softeabi-android";
			case ARM_Linux_SoftEABI_AndroidV7A:
				return "arm-linux-softeabi-androidv7a";
			case ARM_Windows_HardEABI_Default:
				return "arm-windows-hardeabi-default";
			case ARM_NaCl_NaCl_Default:
				return "arm-nacl-nacl-default";
			case MIPS_Linux_O32_Android:
				return "mips-linux-o32-android";
			case IA64_Windows_Microsoft_Default:
				return "ia64-windows-ms-default";
			default:
				return super.toString();
		}
	}

	public OperatingSystem getOperatingSystem() {
		switch (this) {
			case X86_Linux_Pic_i586:
				return OperatingSystem.Linux;
			case X86_Linux_Pic_Android:
				return OperatingSystem.Linux;
			case X86_MacOSX_Pic_Default:
				return OperatingSystem.MacOSX;
			case X86_Windows_Default_i586:
				return OperatingSystem.Windows;
			case X86_NaCl_NaCl_i686:
				return OperatingSystem.NativeClient;
			case X64_Linux_SystemV_Default:
				return OperatingSystem.Linux;
			case X64_Linux_KNC_Default:
				return OperatingSystem.Linux;
			case X64_Windows_Microsoft_Default:
				return OperatingSystem.Windows;
			case X64_MacOSX_SystemV_Default:
				return OperatingSystem.MacOSX;
			case X64_NaCl_NaCl_Default:
				return OperatingSystem.NativeClient;
			case ARM_Linux_OABI_V4T:
				return OperatingSystem.Linux;
			case ARM_Linux_SoftEABI_V5T:
				return OperatingSystem.Linux;
			case ARM_Linux_HardEABI_V7A:
				return OperatingSystem.Linux;
			case ARM_Linux_SoftEABI_Android:
				return OperatingSystem.Linux;
			case ARM_Linux_SoftEABI_AndroidV7A:
				return OperatingSystem.Linux;
			case ARM_Windows_HardEABI_Default:
				return OperatingSystem.Windows;
			case ARM_NaCl_NaCl_Default:
				return OperatingSystem.NativeClient;
			case MIPS_Linux_O32_Android:
				return OperatingSystem.Linux;
			case IA64_Windows_Microsoft_Default:
				return OperatingSystem.Windows;
			default:
				return null;
		}
	}

	public Architecture getArchitecture() {
		switch (this) {
			case X86_Linux_Pic_i586:
				return Architecture.X86;
			case X86_Linux_Pic_Android:
				return Architecture.X86;
			case X86_MacOSX_Pic_Default:
				return Architecture.X86;
			case X86_Windows_Default_i586:
				return Architecture.X86;
			case X86_NaCl_NaCl_i686:
				return Architecture.X86;
			case X64_Linux_SystemV_Default:
				return Architecture.X64;
			case X64_Linux_KNC_Default:
				return Architecture.X64;
			case X64_MacOSX_SystemV_Default:
				return Architecture.X64;
			case X64_Windows_Microsoft_Default:
				return Architecture.X64;
			case X64_NaCl_NaCl_Default:
				return Architecture.X64;
			case ARM_Linux_OABI_V4T:
				return Architecture.ARM;
			case ARM_Linux_SoftEABI_V5T:
				return Architecture.ARM;
			case ARM_Linux_HardEABI_V7A:
				return Architecture.ARM;
			case ARM_Linux_SoftEABI_Android:
				return Architecture.ARM;
			case ARM_Linux_SoftEABI_AndroidV7A:
				return Architecture.ARM;
			case ARM_Windows_HardEABI_Default:
				return Architecture.ARM;
			case ARM_NaCl_NaCl_Default:
				return Architecture.ARM;
			case MIPS_Linux_O32_Android:
				return Architecture.MIPS;
			case IA64_Windows_Microsoft_Default:
				return Architecture.IA64;
			default:
				return null;
		}
	}

}