/*
 *                      EBUILDA: cross-platform build system
 *
 * This file is part of Yeppp! library infrastructure and licensed under Simplified BSD License.
 * See LICENSE.txt for the full text of the license.
 *
 */

package info.yeppp.ebuilda;

import info.yeppp.ebuilda.filesystem.AbsoluteDirectoryPath;
import info.yeppp.ebuilda.filesystem.AbsoluteFilePath;
import info.yeppp.ebuilda.filesystem.RelativeFilePath;
import info.yeppp.ebuilda.generic.CCompiler;
import info.yeppp.ebuilda.generic.CppCompiler;

import java.util.*;

public abstract class AbstractCCompiler implements CCompiler {

	protected AbstractCCompiler(AbstractCCompiler prototype) {
		this.cppCompiler = (AbstractCppCompiler)prototype.cppCompiler.copy();
		this.cppCompiler.setExceptionsSupport(null);
		this.cppCompiler.setRttiEnabled(null);
	}

	protected AbstractCCompiler(AbstractCppCompiler prototype) {
		this.cppCompiler = (AbstractCppCompiler)prototype.copy();
		this.cppCompiler.setExceptionsSupport(null);
		this.cppCompiler.setRttiEnabled(null);
	}

	@Override
	public AbsoluteFilePath getObjectPath(AbsoluteFilePath sourcePath) {
		return this.cppCompiler.getObjectPath(sourcePath);
	}

	@Override
	public void setSourceDirectory(AbsoluteDirectoryPath sourceDirectory) {
		this.cppCompiler.setSourceDirectory(sourceDirectory);
	}

	@Override
	public AbsoluteDirectoryPath getSourceDirectory() {
		return this.cppCompiler.getSourceDirectory();
	}

	@Override
	public void setObjectDirectory(AbsoluteDirectoryPath objectDirectory) {
		this.cppCompiler.setObjectDirectory(objectDirectory);
	}

	@Override
	public AbsoluteDirectoryPath getObjectDirectory() {
		return this.cppCompiler.getObjectDirectory();
	}

	@Override
	public final void setSysRoot(AbsoluteDirectoryPath sysRoot) {
		this.cppCompiler.setSysRoot(sysRoot);
	}

	@Override
	public final AbsoluteDirectoryPath getSysRoot() {
		return this.cppCompiler.getSysRoot();
	}

	@Override
	public void setVerboseBuild(boolean verboseBuild) {
		this.cppCompiler.setVerboseBuild(verboseBuild);
	}

	@Override
	public boolean isVerboseBuild() {
		return this.cppCompiler.isVerboseBuild();
	}

	@Override
	public void setOptimization(Optimization optimization) {
		this.cppCompiler.setOptimization(optimization);
	}

	@Override
	public CCompiler.Optimization getOptimization() {
		return this.cppCompiler.getOptimization();
	}

	@Override
	public void setGlobalOptimization(boolean globalOptimization) {
		this.cppCompiler.setGlobalOptimization(globalOptimization);
	}

	@Override
	public boolean getGlobalOptimization() {
		return this.cppCompiler.getGlobalOptimization();
	}

	@Override
	public void setDebugInformation(boolean debugInformation) {
		this.cppCompiler.setDebugInformation(debugInformation);
	}

	@Override
	public boolean getDebugInformation() {
		return this.cppCompiler.getDebugInformation();
	}

	@Override
	public void setDebugBuild(boolean debugEnabled) {
		this.cppCompiler.setDebugBuild(debugEnabled);
	}

	@Override
	public boolean isDebugBuild() {
		return this.cppCompiler.isDebugBuild();
	}

	@Override
	public void setRuntimeLibrary(RuntimeLibrary runtimeLibrary) {
		this.cppCompiler.setRuntimeLibrary(runtimeLibrary);
	}

	@Override
	public RuntimeLibrary getRuntimeLibrary() {
		return this.cppCompiler.getRuntimeLibrary();
	}

	@Override
	public void setPositionIndependentCodeGeneration(PositionIndependentCodeGeneration pic) {
		this.cppCompiler.setPositionIndependentCodeGeneration(pic);
	}

	@Override
	public PositionIndependentCodeGeneration getPositionIndependentCodeGeneration() {
		return this.cppCompiler.getPositionIndependentCodeGeneration();
	}

	@Override
	public boolean addMacro(String name) {
		return this.addMacro(name, "1");
	}

	@Override
	public boolean addMacro(String name, String value) {
		return this.cppCompiler.addMacro(name, value);
	}

	@Override
	public boolean removeMacro(String name) {
		return this.cppCompiler.removeMacro(name);
	}

	@Override
	public void clearMacros() {
		this.cppCompiler.clearMacros();
	}

	@Override
	public boolean addGlobalIncludeDirectory(AbsoluteDirectoryPath includeDirectory) {
		return this.cppCompiler.addGlobalIncludeDirectory(includeDirectory);
	}

	@Override
	public boolean addGlobalIncludeDirectories(Collection<AbsoluteDirectoryPath> includeDirectories) {
		return this.cppCompiler.addGlobalIncludeDirectories(includeDirectories);
	}

	@Override
	public boolean removeGlobalIncludeDirectory(AbsoluteDirectoryPath includeDirectory) {
		return this.cppCompiler.removeGlobalIncludeDirectory(includeDirectory);
	}

	@Override
	public void clearGlobalIncludeDirectories() {
		this.cppCompiler.clearGlobalIncludeDirectories();
	}

	@Override
	public boolean addIncludeDirectory(AbsoluteDirectoryPath includePath) {
		return this.cppCompiler.addIncludeDirectory(includePath);
	}

	@Override
	public boolean removeIncludeDirectory(AbsoluteDirectoryPath includePath) {
		return this.cppCompiler.removeIncludeDirectory(includePath);
	}

	@Override
	public void clearIncludeDirectories() {
		this.cppCompiler.clearIncludeDirectories();
	}

	@Override
	public void clearAll() {
		this.cppCompiler.clearAll();
	}

	@Override
	public ABI getABI() {
		return this.cppCompiler.getABI();
	}

	final protected AbstractCppCompiler cppCompiler;

}
