/*
 *                      EBUILDA: cross-platform build system
 *
 * This file is part of Yeppp! library infrastructure and licensed under Simplified BSD License.
 * See LICENSE.txt for the full text of the license.
 *
 */

package info.yeppp.ebuilda;

public enum Architecture {
	X86,
	X64,
	IA64,
	ARM,
	MIPS,
	MIPS64,
	PowerPC,
	PowerPC64;

	public final boolean is64bit() {
		switch (this) {
			case X86:
				return false;
			case X64:
				return true;
			case IA64:
				return true;
			case ARM:
				return false;
			case MIPS:
				return false;
			case MIPS64:
				return true;
			case PowerPC:
				return false;
			case PowerPC64:
				return false;
			default:
				throw new Error(String.format("Unknown Architecture value (%s)", this.toString()));
		}
	}

	public final static Architecture detect() {
		final String osArch = System.getProperty("os.arch");
		if (osArch.equals("x86")) {
			if (OperatingSystem.detect().equals(OperatingSystem.Windows)) {
				if (System.getenv("PROCESSOR_ARCHITEW6432") != null) {
					/* 32-bit Java VM runs as a WoW64 process
					 * See http://msdn.microsoft.com/en-us/library/windows/desktop/aa384274.aspx for details.
					 */
					return X64;
				} else {
					return X86;
				}
			} else {
				return X86;
			}
		} else if (osArch.equals("i386")) {
			return X86;
		} else if (osArch.equals("amd64")) {
			return X64;
		} else if (osArch.equals("x86_64")) {
			return X64;
		} else if (osArch.equals("arm")) {
			return ARM;
		} else if (osArch.equals("ppc64")) {
			return PowerPC64;
		} else {
			throw new Error(String.format("Unknown os.arch value (%s)", osArch));
		}
	}

	@Override
	public String toString() {
		switch (this) {
			case X86:
				return "x86";
			case X64:
				return "x86-64";
			default:
				return super.toString();
		}
	}
}
