/*
 *                      EBUILDA: cross-platform build system
 *
 * This file is part of Yeppp! library infrastructure and licensed under Simplified BSD License.
 * See LICENSE.txt for the full text of the license.
 *
 */

package info.yeppp.ebuilda;

import info.yeppp.ebuilda.filesystem.AbsoluteFilePath;

public class BuildMessage {
	public static enum Type {
		FatalError,
		Error,
		Warning,
		Note,
		Exception
	}

	public BuildMessage(Type type, AbsoluteFilePath path, Integer lineNumber, Integer errorNumber, String message) {
		this.type = type;
		this.path = path;
		this.lineNumber = lineNumber;
		this.errorNumber = errorNumber;
		this.message = message;
	}

	public Type getType() {
		return this.type;
	}
	
	public AbsoluteFilePath getPath() {
		return this.path;
	}
	
	public Integer getLineNumber() {
		return this.lineNumber;
	}
	
	public Integer getErrorNumber() {
		return this.errorNumber;
	}
	
	public String getMessage() {
		return this.message;
	}
	
	public void appendMessage(String additionalMessage) {
		this.message += additionalMessage;
	}

	@Override
	public String toString() {
		if (lineNumber != null) {
			return this.path + ": " + this.message + " on line " + this.lineNumber.toString();
		} else if (this.path != null) {
			return this.path + ": " + this.message;
		} else {
			return this.message;
		}
	}

	protected final Type type;
	protected final AbsoluteFilePath path;
	protected final Integer errorNumber;
	protected final Integer lineNumber;
	protected String message;
}
