/*
 *                      EBUILDA: cross-platform build system
 *
 * This file is part of Yeppp! library infrastructure and licensed under Simplified BSD License.
 * See LICENSE.txt for the full text of the license.
 *
 */

package info.yeppp.ebuilda;

import info.yeppp.ebuilda.filesystem.AbsoluteDirectoryPath;
import info.yeppp.ebuilda.filesystem.AbsoluteFilePath;
import info.yeppp.ebuilda.generic.CCompiler;
import info.yeppp.ebuilda.generic.CppCompiler;
import info.yeppp.ebuilda.generic.Versioned;
import info.yeppp.ebuilda.internals.Program;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class GnuCCompiler extends AbstractCCompiler implements Versioned {

	public static ToolInfo verify(AbsoluteFilePath compilerPath) {
		return GnuCppCompiler.verify(compilerPath);
	}

	GnuCCompiler(AbsoluteFilePath compilerPath, ABI abi, Version version, Collection<AbsoluteDirectoryPath> defaultIncludeDirectories, AbsoluteDirectoryPath sysRoot) {
		super(new GnuCppCompiler(compilerPath, abi, version, defaultIncludeDirectories, sysRoot));
	}

	public GnuCCompiler(GnuCCompiler prototype) {
		super(prototype);
	}

	public GccCompatibleCompilerLinker asLinker(Collection<AbsoluteDirectoryPath> defaultGlobalLibraryDirectories) {
		return ((GnuCppCompiler)this.cppCompiler).asLinker(defaultGlobalLibraryDirectories);
	}

	public BuildMessages compile(AbsoluteFilePath source) {
		return this.cppCompiler.compile(source);
	}

	@Override
	public final void addDefaultGlobalIncludeDirectories() {
		this.cppCompiler.addDefaultGlobalIncludeDirectories();
	}

	@Override
	public CCompiler copy() {
		return new GnuCCompiler(this);
	}

	@Override
	public final Version getVersion() {
		return ((GnuCppCompiler)this.cppCompiler).getVersion();
	}

}
