/*
 *                      EBUILDA: cross-platform build system
 *
 * This file is part of Yeppp! library infrastructure and licensed under Simplified BSD License.
 * See LICENSE.txt for the full text of the license.
 *
 */

package info.yeppp.ebuilda;

import info.yeppp.ebuilda.filesystem.AbsoluteFilePath;
import info.yeppp.ebuilda.internals.Program;

public class GnuObjCopy {

	private static class OutputParser implements Program.EventHandler {

		public OutputParser(BuildMessages messages) {
			this.messages = messages;
		}

		private final BuildMessages messages;

		@Override
		public void onStdOutLineWrite(String line) {
			System.out.println(line);
		}

		@Override
		public void onStdErrLineWrite(String line) {
			System.err.println(line);
		}

		@Override
		public void onError(String description) {
			messages.add(new BuildMessage(BuildMessage.Type.Exception, null, null, null, description));
		}

		@Override
		public void onCompletion(Integer status) {
		}
	};


	public GnuObjCopy(AbsoluteFilePath objcopyPath, ABI abi) {
		this.objcopyPath = objcopyPath;
		this.abi = abi;
		this.verboseBuild = false;
	}

	public GnuObjCopy(GnuObjCopy prototype) {
		this.objcopyPath = prototype.objcopyPath;
		this.abi = prototype.abi;
		this.verboseBuild = prototype.verboseBuild;
	}

	public BuildMessages addGnuDebugLink(AbsoluteFilePath binary, AbsoluteFilePath debugInfo) {
		final BuildMessages buildMessages = new BuildMessages();
		final Program.Parameters parameters = new Program.Parameters();
		parameters.add("--add-gnu-debuglink=" + debugInfo.getCanonicalForm());
		parameters.add(binary.getCanonicalForm());
		final Program.Environment environment = new Program.Environment();
		final Program.EventHandler eventHandler = new OutputParser(buildMessages);
		if (isVerboseBuild()) {
			System.out.println(objcopyPath + " " + parameters.toString());
		}
		final Program strip = new Program(objcopyPath, parameters, environment, eventHandler);
		strip.join();
		return buildMessages;
	}

	public void setVerboseBuild(boolean verboseBuild) {
		this.verboseBuild = verboseBuild;
	}

	public boolean isVerboseBuild() {
		return this.verboseBuild;
	}

	public ABI getABI() {
		return abi;
	}

	private final AbsoluteFilePath objcopyPath;
	private final ABI abi;
	private boolean verboseBuild;
}
