/*
 *                      EBUILDA: cross-platform build system
 *
 * This file is part of Yeppp! library infrastructure and licensed under Simplified BSD License.
 * See LICENSE.txt for the full text of the license.
 *
 */

package info.yeppp.ebuilda;

import info.yeppp.ebuilda.filesystem.RelativeDirectoryPath;

import java.util.List;

public class SystemInfo {

	public static void main(String[] args) throws Exception {
		System.out.println("Architecture: " + Architecture.detect().toString());
		System.out.println("Operating system: " + OperatingSystem.detect().toString());
		System.out.println("Application directory: " + Machine.getLocal().getApplicationDirectory().toString());
		System.out.println("User home directory: " + Machine.getLocal().getUserHomeDirectory().toString());
		System.out.println("Working directory: " + Machine.getLocal().getWorkingDirectory().toString());
		System.out.println("Current directory: " + new RelativeDirectoryPath("."));
		ABI[] abis = null;
		if (args.length != 0) {
			abis = new ABI[args.length];
			for (int i = 0; i < args.length; i++) {
				abis[i] = ABI.parse(args[i]);
			}
		} else {
			abis = ABI.values();
		}
		for (final ABI abi : abis) {
			System.out.println("Tools for " + abi.toString());
			final List<WindowsSDK> windowsSDKList = WindowsSDK.enumerate(Machine.getLocal(), abi);
			for (WindowsSDK windowsSDK : windowsSDKList) {
				System.out.println("\t" + windowsSDK.toString());
			}
			final AndroidNDK androidNDK = AndroidNDK.enumerate(Machine.getLocal()).getNewest();
			if (androidNDK != null) {
				final VersionList<AndroidToolchain> androidToolchains = androidNDK.enumerateToolchains(abi, AndroidToolchain.Type.GNU);
				if (androidToolchains.isNonEmpty()) {
					System.out.println("\tAndroid toolchains:");
					for (final AndroidToolchain androidToolchain : androidToolchains.enumerate()) {
						System.out.println("\t\t" + androidToolchain.toString());
					}
				}
			}
			final VersionList<GnuBinutils> gnuBinutilsList = GnuBinutils.enumerate(Machine.getLocal(), abi);
			if (gnuBinutilsList.isNonEmpty()) {
				System.out.println("\tGNU binutils");
				for (final GnuBinutils gnuBinutils : gnuBinutilsList.enumerate()) {
					System.out.println("\t\t" + gnuBinutils.toString());
				}
			}
			final VersionList<GccToolchain> gccToolchainList = GccToolchain.enumerate(Machine.getLocal(), abi);
			if (gccToolchainList.isNonEmpty()) {
				System.out.println("\tGCC toolchains");
				for (final GccToolchain gccToolchain : gccToolchainList.enumerate()) {
					System.out.println("\t\t" + gccToolchain.toString());
				}
			}
			final VersionList<ClangToolchain> clangToolchainList = ClangToolchain.enumerate(Machine.getLocal(), abi);
			if (clangToolchainList.isNonEmpty()) {
				System.out.println("\tClang toolchains");
				for (final ClangToolchain clangToolchain : clangToolchainList.enumerate()) {
					System.out.println("\t\t" + clangToolchain.toString());
				}
			}
			final VersionList<IntelCppToolchain> intelCppToolchainList = IntelCppToolchain.enumerate(Machine.getLocal(), abi);
			if (intelCppToolchainList.isNonEmpty()) {
				System.out.println("\tIntel C/C++ toolchains");
				for (final IntelCppToolchain intelCppToolchain : intelCppToolchainList.enumerate()) {
					System.out.println("\t\t" + intelCppToolchain.toString());
				}
			}
			final VersionList<NASM> nasmVersionList = NASM.enumerate(Machine.getLocal(), abi);
			if (nasmVersionList.isNonEmpty()) {
				System.out.println("\tNetwide Assembler:");
				for (final NASM nasm : nasmVersionList.enumerate()) {
					System.out.println("\t\t" + nasm.toString());
				}
			}
			final VersionList<AppleDSymUtil> appleDSymUtilList = AppleDSymUtil.enumerate(Machine.getLocal(), abi);
			if (appleDSymUtilList.isNonEmpty()) {
				System.out.println("\tApple dsymutil:");
				for (final AppleDSymUtil appleDSymUtil : appleDSymUtilList.enumerate()) {
					System.out.println("\t\t" + appleDSymUtil.toString());
				}
			}
			final VersionList<AppleStrip> appleStripList = AppleStrip.enumerate(Machine.getLocal(), abi);
			if (appleStripList.isNonEmpty()) {
				System.out.println("\tApple strip:");
				for (final AppleStrip appleStrip : appleStripList.enumerate()) {
					System.out.println("\t\t" + appleStrip.toString());
				}
			}
			final VersionList<JavaSDK> javaSDKList = JavaSDK.enumerate(Machine.getLocal());
			if (javaSDKList.isNonEmpty()) {
				System.out.println("\tJava SDK:");
				for (final JavaSDK sdk : javaSDKList.enumerate()) {
					System.out.println("\t\t" + sdk.toString());
				}
			}
		}
	}

}
