/*
 *                      EBUILDA: cross-platform build system
 *
 * This file is part of Yeppp! library infrastructure and licensed under Simplified BSD License.
 * See LICENSE.txt for the full text of the license.
 *
 */

package info.yeppp.ebuilda;

import info.yeppp.ebuilda.generic.Versioned;

import java.util.*;

public class VersionList<T extends Versioned> {

	public VersionList() {
		this.items = new LinkedList<T>();
	}

	public VersionList(Collection<T> items) {
		this.items = new ArrayList<T>(items);
	}

	public void add(T item) {
		this.items.add(item);
	}

	public void add(VersionList<T> versionList) {
		this.items.addAll(versionList.items);
	}

	public void removeOld(Version minVersion) {
		final Iterator<T> iterator = items.iterator();
		while (iterator.hasNext()) {
			final T item = iterator.next();
			final Version itemVersion = item.getVersion();
			if (itemVersion.compareTo(minVersion) < 0) {
				iterator.remove();
			}
		}
	}

	public T getNewest() {
		T newestItem = null;
		for (final T item : items) {
			if (newestItem == null) {
				newestItem = item;
			} else if (item.getVersion().compareTo(newestItem.getVersion()) > 0) {
				newestItem = item;
			}
		}
		return newestItem;
	}

	public T getOldest() {
		T oldestItem = null;
		for (final T item : items) {
			if (oldestItem == null) {
				oldestItem = item;
			} else if (item.getVersion().compareTo(oldestItem.getVersion()) < 0) {
				oldestItem = item;
			}
		}
		return oldestItem;
	}

	public T getOldest(Version minVersion) {
		T oldestItem = null;
		for (final T item : items) {
			if (item.getVersion().compareTo(minVersion) >= 0) {
				if (oldestItem == null) {
					oldestItem = item;
				} else if (item.getVersion().compareTo(oldestItem.getVersion()) < 0) {
					oldestItem = item;
				}
			}
		}
		return oldestItem;
	}

	public Collection<T> enumerate() {
		return items;
	}

	final public boolean isEmpty() {
		return items.isEmpty();
	}

	final public boolean isNonEmpty() {
		return !isEmpty();
	}

	private final List<T> items;
}
