/*
 *                      EBUILDA: cross-platform build system
 *
 * This file is part of Yeppp! library infrastructure and licensed under Simplified BSD License.
 * See LICENSE.txt for the full text of the license.
 *
 */

package info.yeppp.ebuilda;

import info.yeppp.ebuilda.ABI;
import info.yeppp.ebuilda.Machine;
import info.yeppp.ebuilda.MicrosoftResourceCompiler;
import info.yeppp.ebuilda.Version;
import info.yeppp.ebuilda.filesystem.AbsoluteDirectoryPath;
import info.yeppp.ebuilda.filesystem.AbsoluteFilePath;
import info.yeppp.ebuilda.filesystem.RelativeDirectoryPath;
import info.yeppp.ebuilda.filesystem.RelativeFilePath;

import java.util.*;

public abstract class WindowsSDK {

	public static List<WindowsSDK> enumerate(Machine machine, ABI abi) {
		final List<WindowsSDK> implementations = new LinkedList<WindowsSDK>();
		implementations.addAll(Windows8SDK.enumerate(machine, abi));
		implementations.addAll(Windows7SDK.enumerate(machine, abi));
		return implementations;
	}

	protected WindowsSDK(AbsoluteDirectoryPath rootDirectory, AbsoluteDirectoryPath includeDirectory, AbsoluteDirectoryPath libraryDirectory, ABI abi, Version version) {
		this.rootDirectory = rootDirectory;
		this.includeDirectory = includeDirectory;
		this.libraryDirectory = libraryDirectory;
		this.abi = abi;
		this.version = version;
	}

	public AbsoluteDirectoryPath getRootDirectory() {
		return this.rootDirectory;
	}

	public Collection<AbsoluteDirectoryPath> getIncludeDirectories() {
		final List<AbsoluteDirectoryPath> includeDirectories = new LinkedList<AbsoluteDirectoryPath>();
		includeDirectories.add(this.includeDirectory);
		return includeDirectories;
	}

	public Collection<AbsoluteDirectoryPath> getLibraryDirectories() {
		final List<AbsoluteDirectoryPath> libraryDirectories = new LinkedList<AbsoluteDirectoryPath>();
		libraryDirectories.add(this.libraryDirectory);
		return libraryDirectories;
	}

	public AbsoluteDirectoryPath getToolsDirectory() {
		return new AbsoluteDirectoryPath(this.rootDirectory, new RelativeDirectoryPath("bin"));
	}

	protected AbsoluteFilePath getResourceCompilerPath() {
		return new AbsoluteFilePath(getToolsDirectory(), new RelativeFilePath("rc.exe"));
	}

	public MicrosoftResourceCompiler getResourceCompiler() {
		final List<AbsoluteDirectoryPath> additionalPathDirectories = new ArrayList<AbsoluteDirectoryPath>(1);
		additionalPathDirectories.add(getToolsDirectory());
		return new MicrosoftResourceCompiler(getResourceCompilerPath(), additionalPathDirectories, getIncludeDirectories());
	}

	public ABI getABI() {
		return this.abi;
	}

	@Override
	public String toString() {
		return String.format("Windows SDK version %s for %s", this.version.toString(), this.abi.toString());
	}

	protected final ABI abi;
	protected final Version version;
	protected final AbsoluteDirectoryPath rootDirectory;
	protected final AbsoluteDirectoryPath includeDirectory;
	protected final AbsoluteDirectoryPath libraryDirectory;
}
