/*
 *                      EBUILDA: cross-platform build system
 *
 * This file is part of Yeppp! library infrastructure and licensed under Simplified BSD License.
 * See LICENSE.txt for the full text of the license.
 *
 */

package info.yeppp.ebuilda.generic;

import info.yeppp.ebuilda.ABI;
import info.yeppp.ebuilda.BuildMessages;
import info.yeppp.ebuilda.PositionIndependentCodeGeneration;
import info.yeppp.ebuilda.filesystem.AbsoluteDirectoryPath;
import info.yeppp.ebuilda.filesystem.AbsoluteFilePath;

import java.util.Collection;

public interface CppCompiler {

	public static enum Exceptions {
		NoExceptions,
		DisabledExceptions,
		SyncronousExceptions,
		AsyncronousExceptions
	}

	public BuildMessages compile(AbsoluteFilePath sourcePath);

	public AbsoluteFilePath getObjectPath(AbsoluteFilePath sourcePath);

	public void setSourceDirectory(AbsoluteDirectoryPath sourceDirectory);

	public AbsoluteDirectoryPath getSourceDirectory();

	public void setObjectDirectory(AbsoluteDirectoryPath objectDirectory);

	public AbsoluteDirectoryPath getObjectDirectory();

	public void setSysRoot(AbsoluteDirectoryPath sysRoot);

	public AbsoluteDirectoryPath getSysRoot();

	public void setVerboseBuild(boolean verboseBuild);

	public boolean isVerboseBuild();

	public void setOptimization(CCompiler.Optimization optimization);

	public CCompiler.Optimization getOptimization();

	public void setGlobalOptimization(boolean globalOptimization);

	public boolean getGlobalOptimization();

	public void setDebugInformation(boolean debugInformation);

	public boolean getDebugInformation();

	public void setDebugBuild(boolean debugBuild);

	public boolean isDebugBuild();

	public void setPositionIndependentCodeGeneration(PositionIndependentCodeGeneration pic);

	public PositionIndependentCodeGeneration getPositionIndependentCodeGeneration();

	public void setRuntimeLibrary(CCompiler.RuntimeLibrary runtimeLibrary);

	public CCompiler.RuntimeLibrary getRuntimeLibrary();

	public void addDefaultGlobalIncludeDirectories();

	public boolean addGlobalIncludeDirectory(AbsoluteDirectoryPath includeDirectory);

	public boolean addGlobalIncludeDirectories(Collection<AbsoluteDirectoryPath> includeDirectories);

	public boolean removeGlobalIncludeDirectory(AbsoluteDirectoryPath includePath);

	public void clearGlobalIncludeDirectories();

	public boolean addIncludeDirectory(AbsoluteDirectoryPath includePath);

	public boolean removeIncludeDirectory(AbsoluteDirectoryPath includePath);

	public void clearIncludeDirectories();

	public boolean addMacro(String name);

	public boolean addMacro(String name, String value);

	public boolean removeMacro(String name);

	public void clearMacros();

	public void clearAll();

	public ABI getABI();

	public void setExceptionsSupport(Exceptions exceptions);

	public Exceptions getExceptionsEnabled();

	public void setRttiEnabled(Boolean rttiEnabled);

	public Boolean isRttiEnabled();

	public CppCompiler copy();

}
